@@ -10,7 +10,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "lib.h"
@@ -54,6 +54,7 @@
 #endif
 
 static DM_LIST_INIT(_open_devices);
+static unsigned _dev_size_seqno = 1;
 
 /*-----------------------------------------------------------------
  * The standard io loop that keeps submitting an io until it's
@@ -271,11 +272,18 @@ out:
 	return r;
 }
 
-static int _dev_get_size_file(const struct device *dev, uint64_t *size)
+static int _dev_get_size_file(struct device *dev, uint64_t *size)
 {
 	const char *name = dev_name(dev);
 	struct stat info;
 
+	if (dev->size_seqno == _dev_size_seqno) {
+		log_very_verbose("%s: using cached size %" PRIu64 " sectors",
+				 name, dev->size);
+		*size = dev->size;
+		return 1;
+	}
+
 	if (stat(name, &info)) {
 		log_sys_error("stat", name);
 		return 0;
@@ -283,6 +291,8 @@ static int _dev_get_size_file(const struct device *dev, uint64_t *size)
 
 	*size = info.st_size;
 	*size >>= SECTOR_SHIFT;	/* Convert to sectors */
+	dev->size = *size;
+	dev->size_seqno = _dev_size_seqno;
 
 	log_very_verbose("%s: size is %" PRIu64 " sectors", name, *size);
 
@@ -293,6 +303,13 @@ static int _dev_get_size_dev(struct device *dev, uint64_t *size)
 {
 	const char *name = dev_name(dev);
 
+	if (dev->size_seqno == _dev_size_seqno) {
+		log_very_verbose("%s: using cached size %" PRIu64 " sectors",
+				 name, dev->size);
+		*size = dev->size;
+		return 1;
+	}
+
 	if (!dev_open_readonly(dev))
 		return_0;
 
@@ -304,6 +321,9 @@ static int _dev_get_size_dev(struct device *dev, uint64_t *size)
 	}
 
 	*size >>= BLKSIZE_SHIFT;	/* Convert to sectors */
+	dev->size = *size;
+	dev->size_seqno = _dev_size_seqno;
+
 	if (!dev_close(dev))
 		log_sys_error("close", name);
 
@@ -373,6 +393,10 @@ static int _dev_discard_blocks(struct device *dev, uint64_t offset_bytes, uint64
 /*-----------------------------------------------------------------
  * Public functions
  *---------------------------------------------------------------*/
+void dev_size_seqno_inc(void)
+{
+	_dev_size_seqno++;
+}
 
 int dev_get_size(struct device *dev, uint64_t *size)
 {
@@ -470,11 +494,22 @@ int dev_open_flags(struct device *dev, int flags, int direct, int quiet)
 
 #ifdef O_NOATIME
 	/* Don't update atime on device inodes */
-	if (!(dev->flags & DEV_REGULAR))
+	if (!(dev->flags & DEV_REGULAR) && !(dev->flags & DEV_NOT_O_NOATIME))
 		flags |= O_NOATIME;
 #endif
 
 	if ((dev->fd = open(name, flags, 0777)) < 0) {
+#ifdef O_NOATIME
+		if ((errno == EPERM) && (flags & O_NOATIME)) {
+			flags &= ~O_NOATIME;
+			dev->flags |= DEV_NOT_O_NOATIME;
+			if ((dev->fd = open(name, flags, 0777)) >= 0) {
+				log_debug_devs("%s: Not using O_NOATIME", name);
+				goto opened;
+			}
+		}
+#endif
+
 #ifdef O_DIRECT_SUPPORT
 		if (direct && !(dev->flags & DEV_O_DIRECT_TESTED)) {
 			flags &= ~O_DIRECT;
@@ -489,6 +524,8 @@ int dev_open_flags(struct device *dev, int flags, int direct, int quiet)
 			log_sys_debug("open", name);
 		else
 			log_sys_error("open", name);
+
+		dev->flags |= DEV_OPEN_FAILURE;
 		return 0;
 	}
 
@@ -532,6 +569,7 @@ int dev_open_flags(struct device *dev, int flags, int direct, int quiet)
 		       dev->flags & DEV_OPENED_EXCL ? " O_EXCL" : "",
 		       dev->flags & DEV_O_DIRECT ? " O_DIRECT" : "");
 
+	dev->flags &= ~DEV_OPEN_FAILURE;
 	return 1;
 }
 
