@@ -169,7 +169,7 @@ prepare_clvmd() {
 	rm -f "$CLVMD_PIDFILE"
 	echo "<======== Starting CLVMD ========>"
 	# lvs is executed from clvmd - use our version
-	LVM_LOG_FILE_EPOCH=CLVMD LVM_BINARY=$(which lvm) $run_valgrind clvmd -Isinglenode -d 1 -f &
+	LVM_LOG_FILE_EPOCH=CLVMD LVM_LOG_FILE_MAX_LINES=1000000 LVM_BINARY=$(which lvm) $run_valgrind clvmd -Isinglenode -d 1 -f &
 	echo $! > LOCAL_CLVMD
 
 	for i in {1..100} ; do
@@ -325,11 +325,10 @@ prepare_lvmdbusd() {
 		daemon="$(which lvmdbusd || :)"
 	fi
 	[[ -n $daemon && -x $daemon ]] || skip "The daemon is missing"
-
 	which python3 >/dev/null || skip "Missing python3"
+
 	python3 -c "import pyudev, dbus, gi.repository" || skip "Missing python modules"
 
-        # TODO: Tests should use session bus instead of system bus
 	# Copy the needed file to run on the system bus if it doesn't
 	# already exist
 	if [ ! -f /etc/dbus-1/system.d/com.redhat.lvmdbus1.conf ]; then
@@ -337,7 +336,9 @@ prepare_lvmdbusd() {
 	fi
 
 	echo "preparing lvmdbusd..."
-	"$daemon" --debug --udev > debug.log_LVMDBUSD_out 2>&1 &
+	lvmconf "global/notify_dbus = 1"
+
+	"$daemon" --debug  > debug.log_LVMDBUSD_out 2>&1 &
 	local pid=$!
 
 	sleep 1
@@ -1062,7 +1063,7 @@ generate_config() {
             cat > "$config_values" <<-EOF
 activation/checks = 1
 activation/monitoring = 0
-activation/polling_interval = 0
+activation/polling_interval = 1
 activation/retry_deactivation = 1
 activation/snapshot_autoextend_percent = 50
 activation/snapshot_autoextend_threshold = 50
@@ -1080,6 +1081,7 @@ devices/global_filter = [ "a|$DM_DEV_DIR/mapper/.*pv[0-9_]*$|", "r|.*|" ]
 devices/md_component_detection  = 0
 devices/scan = "$DM_DEV_DIR"
 devices/sysfs_scan = 1
+devices/write_cache_state = 0
 global/abort_on_internal_errors = 1
 global/cache_check_executable = "$LVM_TEST_CACHE_CHECK_CMD"
 global/cache_dump_executable = "$LVM_TEST_CACHE_DUMP_CMD"
@@ -1089,6 +1091,7 @@ global/fallback_to_local_locking = 0
 global/library_dir = "$TESTDIR/lib"
 global/locking_dir = "$TESTDIR/var/lock/lvm"
 global/locking_type=$LVM_TEST_LOCKING
+global/notify_dbus = 0
 global/si_unit_consistency = 1
 global/thin_check_executable = "$LVM_TEST_THIN_CHECK_CMD"
 global/thin_dump_executable = "$LVM_TEST_THIN_DUMP_CMD"
@@ -1373,6 +1376,15 @@ have_raid() {
 	esac
 }
 
+have_raid4 () {
+	local r=1
+
+	have_raid 1 8 0 && r=0
+	have_raid 1 9 1 && r=1
+
+	return $r
+}
+
 have_cache() {
 	test "$CACHE" = shared -o "$CACHE" = internal || {
 		echo "Cache is not built-in." >&2
