@@ -10,7 +10,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "tools.h"
@@ -40,10 +40,10 @@ progress_t poll_mirror_progress(struct cmd_context *cmd,
 
 	overall_percent = copy_percent(lv);
 	if (parms->progress_display)
-		log_print_unless_silent("%s: %s: %.1f%%", name, parms->progress_title,
+		log_print_unless_silent("%s: %s: %.2f%%", name, parms->progress_title,
 					dm_percent_to_float(overall_percent));
 	else
-		log_verbose("%s: %s: %.1f%%", name, parms->progress_title,
+		log_verbose("%s: %s: %.2f%%", name, parms->progress_title,
 			    dm_percent_to_float(overall_percent));
 
 	if (segment_percent != DM_PERCENT_100)
@@ -242,22 +242,22 @@ static struct poll_operation_id *copy_poll_operation_id(struct dm_pool *mem,
 {
 	struct poll_operation_id *copy;
 
-	if (!id)
-		return_NULL;
+	if (!id || !id->vg_name || !id->lv_name || !id->display_name || !id->uuid) {
+		log_error(INTERNAL_ERROR "Wrong params for copy_poll_operation_id.");
+		return NULL;
+	}
 
-	copy = (struct poll_operation_id *) dm_pool_alloc(mem, sizeof(struct poll_operation_id));
-	if (!copy) {
+	if (!(copy = dm_pool_alloc(mem, sizeof(*copy)))) {
 		log_error("Poll operation ID allocation failed.");
 		return NULL;
 	}
 
-	copy->display_name = id->display_name ? dm_pool_strdup(mem, id->display_name) : NULL;
-	copy->lv_name = id->lv_name ? dm_pool_strdup(mem, id->lv_name) : NULL;
-	copy->vg_name = id->vg_name ? dm_pool_strdup(mem, id->vg_name) : NULL;
-	copy->uuid = id->uuid ? dm_pool_strdup(mem, id->uuid) : NULL;
-
-	if (!copy->display_name || !copy->lv_name || !copy->vg_name || !copy->uuid) {
+	if (!(copy->display_name = dm_pool_strdup(mem, id->display_name)) ||
+	    !(copy->lv_name = dm_pool_strdup(mem, id->lv_name)) ||
+	    !(copy->vg_name = dm_pool_strdup(mem, id->vg_name)) ||
+	    !(copy->uuid = dm_pool_strdup(mem, id->uuid))) {
 		log_error("Failed to copy one or more poll_operation_id members.");
+		dm_pool_free(mem, copy);
 		return NULL;
 	}
 
@@ -364,7 +364,7 @@ static void _poll_for_all_vgs(struct cmd_context *cmd,
 
 	while (1) {
 		parms->outstanding_count = 0;
-		process_each_vg(cmd, 0, NULL, READ_FOR_UPDATE, handle, _poll_vg);
+		process_each_vg(cmd, 0, NULL, NULL, NULL, READ_FOR_UPDATE, 0, handle, _poll_vg);
 		if (!parms->outstanding_count)
 			break;
 		if (parms->interval)
@@ -508,7 +508,7 @@ static void _lvmpolld_poll_for_all_vgs(struct cmd_context *cmd,
 
 	handle->custom_handle = &lpdp;
 
-	process_each_vg(cmd, 0, NULL, 0, handle, _lvmpolld_init_poll_vg);
+	process_each_vg(cmd, 0, NULL, NULL, NULL, 0, 0, handle, _lvmpolld_init_poll_vg);
 
 	first = dm_list_first(&lpdp.idls);
 
@@ -561,7 +561,7 @@ static int _lvmpoll_daemon(struct cmd_context *cmd, struct poll_operation_id *id
 		return r ? ECMD_PROCESSED : ECMD_FAILED;
 	} else {
 		/* process all in-flight operations */
-		if (!(handle = init_processing_handle(cmd))) {
+		if (!(handle = init_processing_handle(cmd, NULL))) {
 			log_error("Failed to initialize processing handle.");
 			return ECMD_FAILED;
 		} else {
@@ -596,6 +596,11 @@ static int _poll_daemon(struct cmd_context *cmd, struct poll_operation_id *id,
 			parms->progress_display = 0; /* Child */
 		/* FIXME Use wait_event (i.e. interval = 0) and */
 		/*       fork one daemon per copy? */
+
+		if ((daemon_mode == 1) && find_config_tree_bool(cmd, global_use_lvmetad_CFG, NULL)) {
+			if (!lvmetad_connect(cmd))
+				log_warn("WARNING: lvm polling process %d cannot connect to lvmetad.", getpid());
+		}
 	}
 
 	/*
@@ -609,7 +614,7 @@ static int _poll_daemon(struct cmd_context *cmd, struct poll_operation_id *id,
 	} else {
 		if (!parms->interval)
 			parms->interval = find_config_tree_int(cmd, activation_polling_interval_CFG, NULL);
-		if (!(handle = init_processing_handle(cmd))) {
+		if (!(handle = init_processing_handle(cmd, NULL))) {
 			log_error("Failed to initialize processing handle.");
 			ret = ECMD_FAILED;
 		} else {
